#ifndef _DEF_H_
#define _DEF_H_

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <locale.h>
#include <time.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/XKBlib.h>
#include <X11/Xft/Xft.h>
#include <X11/extensions/Xinerama.h>
#include <Imlib2.h>

/* macros */
#define MAXPATHS 128            /* maximal number of paths to look for icons */
#define ICONPATH "ICONPATH"     /* environment variable name */
#define CLASS               "XMenu"
#define LEN(x)              (sizeof (x) / sizeof (x[0]))
#define BETWEEN(x, a, b)    ((a) <= (x) && (x) <= (b))
#define GETNUM(n, s) { \
	unsigned long __TMP__; \
	if ((__TMP__ = strtoul((s), NULL, 10)) < INT_MAX) \
		(n) = __TMP__; \
	}

/* Actions for the main loop */
enum {
	ACTION_NOP    = 0,
	ACTION_CLEAR  = 1<<0,     /* clear text */
	ACTION_SELECT = 1<<1,     /* select item */
	ACTION_MAP    = 1<<2,     /* remap menu windows */
	ACTION_DRAW   = 1<<3,     /* redraw menu windows */
	ACTION_WARP   = 1<<4,     /* warp the pointer */
};

/* enum for keyboard menu navigation */
enum { ITEMPREV, ITEMNEXT, ITEMFIRST, ITEMLAST };

/* enum for text alignment */
enum {LeftAlignment, CenterAlignment, RightAlignment};

/* color enum */
enum {ColorFG, ColorBG, ColorLast};

/* EWMH atoms */
enum {NetWMName, NetWMWindowType, NetWMWindowTypePopupMenu, NetLast};


extern int m_tty;   /* tty to which the output commands might be redirected */

/* flags */
extern int iflag;                   /* whether to disable icons */
extern int rflag;                   /* whether to disable right-click */
extern int mflag;                   /* whether the user specified a monitor with -p */
extern int pflag;                   /* whether the user specified a position with -p */
extern int wflag;                   /* whether to let the window manager control XMenu */
extern int eflag;                   /* Whether to run as an executor plugin for tint2 */
extern int rootmodeflag;            /* whether to run in root mode */
extern int passclickflag;           /* whether to pass click to root window */
extern int firsttime;               /* set to 0 after first run */

/* arguments */
extern unsigned int button;         /* button to trigger pmenu in root mode */
extern unsigned int modifier;       /* modifier to trigger pmenu */

/* icons paths */
extern char *iconstring;         /* string read from getenv */
extern char *iconpaths[MAXPATHS];       /* paths to icon directories */
extern int niconpaths;              /* number of paths to icon directories */

/*
 * KEYBINDINGS
 *
 * Look at your /usr/include/X11/keysymdef.h  (or the equivalent file
 * in your system) for a list of key symbol constants, and change the
 * macros below accordingly.  All key symbol constants begin with the
 * prefix XK_.
 *
 * For example, to use vim-like key bindings, set KEYSYMLEFT to XK_h,
 * KEYSYMDOWN to XK_j, KEYSYMUP to XK_k, etc.
 *
 * Note that the regular keys like ArrowUp, ArrowDown, Tab, Home, etc
 * will ALWAYS work, so you do not need to set them.
 *
 * If you do not want to set a key binding, keep it with the value of
 * XK_VoidSymbol
 */
#define KSYMFIRST   XK_VoidSymbol       /* select first item */
#define KSYMLAST    XK_VoidSymbol       /* select last item */
#define KSYMUP      XK_VoidSymbol       /* select previous item */
#define KSYMDOWN    XK_VoidSymbol       /* select next item */
#define KSYMLEFT    XK_VoidSymbol       /* close current menu */
#define KSYMRIGHT   XK_VoidSymbol       /* enter selected item */

/* draw context structure */
struct DC {
	XftColor normal[ColorLast];
	XftColor selected[ColorLast];
	XftColor border;
	XftColor separator;

	GC gc;

	FcPattern *pattern;
	XftFont **fonts;
	size_t nfonts;
};

/* menu item structure */
struct Item {
	char *label;            /* string to be drawed on menu */
	char *output;           /* string to be outputed when item is clicked */
	char *cmd;              /* command to redirect the output string to a concrete tty */ 
	char *file;             /* filename of the icon */
	int y;                  /* item y position relative to menu */
	int h;                  /* item height */
	int textw;              /* text width */
	struct Item *prev;      /* previous item */
	struct Item *next;      /* next item */
	struct Menu *submenu;   /* submenu spawned by clicking on item */
	Drawable sel, unsel;    /* pixmap for selected and unselected item */
	Imlib_Image icon;
};

/* monitor geometry structure */
struct Monitor {
	int x, y, w, h;         /* monitor geometry */
};

/* menu structure */
struct Menu {
	struct Menu *parent;    /* parent menu */
	struct Item *caller;    /* item that spawned the menu */
	struct Item *list;      /* list of items contained by the menu */
	struct Item *first;     /* first item displayed on the menu */
	struct Item *selected;  /* item currently selected in the menu */
	int x, y, w, h;         /* menu geometry */
	int drawn;              /* whether the menu was already drawn */
	int maxtextw;           /* maximum text width */
	int level;              /* menu level relative to root */
	int overflow;           /* whether the menu is higher than the monitor */
	Window win;             /* menu window to map on the screen */
	XIC xic;                /* input context */
};

/* X stuff */
extern Display *dpy;
extern Visual *visual;
extern Window rootwin;
extern Colormap colormap;
extern XrmDatabase xdb;
extern XClassHint classh;
extern char *xrm;
extern struct DC dc;
extern Atom utf8string;
extern Atom wmdelete;
extern Atom netatom[NetLast];
extern int screen;
extern int depth;
extern XIM xim;

#define strdup_or_null( str )  (str) != NULL ? strdup(str) : NULL

#endif // _DEF_H_
